#include <LinuxProcess.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <string.h>
#include <dirent.h>
#include <errno.h>
#include <pwd.h>
#include <grp.h>

LinuxProcess::LinuxProcess ( int procID )
	: next(NULL)
	{
	cmdline       = NULL;	
	pid           = procID;		
	execName      = NULL;		
	state         = 0;	
	parent        = 0;	
	uid           = 0;
	user          = NULL;
	gid           = 0;
	group         = NULL;

	char        strBuf [4096];
	char        fileBuf[4096];	
	struct stat statBuf;
	FILE      * fp ;
	
	sprintf(fileBuf, "/proc/%i", pid);
	if(!stat(fileBuf, &statBuf))
		{
		uid = statBuf.st_uid;
		struct passwd * pw = getpwuid(uid);
		if(pw && *pw->pw_name) user = strdup(pw->pw_name);
		else user = strdup("\0");

		gid = statBuf.st_gid;
		struct group * gr = getgrgid(gid);
		if(gr && *gr->gr_name) group = strdup(gr->gr_name);
		else group = strdup("\0");
		}
	else 	{
		user = strdup("\0");
		group = strdup("\0");
		}
		
	sprintf(fileBuf, "/proc/%i/cmdline", pid);
	if((fp = fopen(fileBuf, "r"))!=NULL)
		{
		int amntRead = 0;
		if((amntRead = fread(strBuf, 1, 4096, fp))>0) 
			{
			strBuf[amntRead]=0;
			while(--amntRead>=0)
				{
				if(strBuf[amntRead]==0) strBuf[amntRead]=32;
				}
			cmdline = strdup(strBuf);
			}
		else cmdline = strdup("\0");
		fclose(fp);
		}
	else cmdline = strdup("\0");
	
	sprintf(fileBuf, "/proc/%i/stat", pid);
	if((fp = fopen(fileBuf, "r"))!=NULL)
		{
		fscanf(fp, "%d %s %c %d", &pid, strBuf, &state, &parent);
		char * ptr = strBuf;
		if(*(ptr+strlen(ptr)-1)==')')
			{
			ptr++;
			*(ptr+strlen(ptr)-1) = 0;
			}
		execName = strdup(ptr);
		}
	else execName = strdup("\0");
	}
	
LinuxProcess::~LinuxProcess ( void )
	{
	if(cmdline)  free(cmdline);
	if(execName) free(execName);
	if(user)     free(user);
	if(group)    free(group);
	}

void LinuxProcess::asciiDump ( FILE * fp )
	{
	fprintf(fp, "Process             : %i\n", pid);
	fprintf(fp, "   Command Line     : %s\n", (cmdline && *cmdline)?cmdline:"(Unknown)");
	fprintf(fp, "   Executable Name  : %s\n", (execName && *execName)?execName:"(Unknown)");
	fprintf(fp, "   State            : %c\n", state);
	fprintf(fp, "   Parent Process   : %i\n", parent);
	fprintf(fp, "   Owning User      : %s (%i)\n", (user && *user)?user:"(Unknown)", uid);
	fprintf(fp, "   Owning Group     : %s (%i)\n\n", (group && *group)?group:"(Unknown)", gid);
	}
	

LinuxProcessReply::LinuxProcessReply ( void )
	: count (0),
	  max   (16),
	  items ((LinuxProcess **)malloc(16*sizeof(LinuxProcess *)))
	{
	}	  

LinuxProcessReply::~LinuxProcessReply ( void )
	{
	free (items);
	}

void LinuxProcessReply::addProcess ( LinuxProcess * item )
	{
	if(item)
		{
		if(count+1 >= max)
			{
			max *= 2;
			items = (LinuxProcess **)realloc(items, max*sizeof(LinuxProcess *));
			}
		items[count++] = item;
		}
	}

LinuxProcess * LinuxProcessReply::operator [] (size_t idx )
	{
	return (idx<count)?items[idx]:NULL;
	}

void LinuxProcessReply::asciiDump ( FILE * fp )
	{
	for(int idx=0; idx<count; idx++)
		{
		items[idx]->asciiDump(fp);
		}
	}	
	
LinuxProcessList::LinuxProcessList ( void )
	: procs(NULL), tail(NULL)
	{
	DIR    * dir = opendir("/proc");
	dirent * entry;
	int      pid;
	
	if(dir)	{
		while((entry=readdir(dir))!=NULL)
			{
			if((pid=atoi(entry->d_name))!=0)
				{
				LinuxProcess * proc = new LinuxProcess(pid);
				if(proc->cmdline  &&
				   proc->execName &&
				   proc->user     &&
				   proc->group )
					{
					if(tail) tail->setNext(proc);
					else     procs = proc;
					tail = proc;
					}
				else delete proc;
				}
			}
		closedir(dir);
		}
	}
	
LinuxProcessList::~LinuxProcessList ( void )
	{
	while(procs!=NULL)
		{
		LinuxProcess * proc = procs;
		procs = proc->getNext();
		delete proc;
		}
	}
	
LinuxProcessReply * LinuxProcessList::getProc        ( void )
	{
	LinuxProcessReply * reply = new LinuxProcessReply();
	LinuxProcess * proc = procs;
	while(proc!=NULL)
		{
		reply->addProcess(proc);
		proc = proc->getNext();
		}
	return reply;
	}
	
LinuxProcessReply * LinuxProcessList::getProcByPid   ( pid_t pid )
	{
	LinuxProcessReply * reply = new LinuxProcessReply();
	LinuxProcess * proc = procs;
	while(proc!=NULL)
		{
		if(proc->pid == pid) reply->addProcess(proc);
		proc = proc->getNext();
		}
	return reply;
	}	
	
LinuxProcessReply * LinuxProcessList::getProcByName  ( char * name )
	{
	LinuxProcessReply * reply = new LinuxProcessReply();
	LinuxProcess * proc = procs;
	char tStr[4096];
	char * ptr;
	while(proc!=NULL)
		{
		strncpy(tStr, proc->cmdline, 4096);
		tStr[4095]=0;
		if((ptr = strchr(tStr, ' '))!=NULL) *ptr = 0;
		if(!strcmp(tStr, name) || !strcmp(proc->execName, name))
			{
			reply->addProcess(proc);
			}
		proc = proc->getNext();
		}
	return reply;
	}
	
LinuxProcessReply * LinuxProcessList::getProcByUser  ( uid_t uid )
	{
	LinuxProcessReply * reply = new LinuxProcessReply();
	LinuxProcess * proc = procs;
	while(proc!=NULL)
		{
		if(proc->uid == uid) reply->addProcess(proc);
		proc = proc->getNext();
		}
	return reply;
	}
	
LinuxProcessReply * LinuxProcessList::getProcByUser  ( char * name )
	{
	struct passwd     * pwd   = getpwnam(name);
	return (pwd)?getProcByUser(pwd->pw_uid):new LinuxProcessReply;
	}
	
LinuxProcessReply * LinuxProcessList::getProcByGroup ( gid_t gid )
	{
	LinuxProcessReply * reply = new LinuxProcessReply();
	LinuxProcess * proc = procs;
	while(proc!=NULL)
		{
		if(proc->gid == gid) reply->addProcess(proc);
		proc = proc->getNext();
		}
	return reply;
	}
	
LinuxProcessReply * LinuxProcessList::getProcByGroup ( char * name )
	{
	struct group      * gr   = getgrnam(name);
	return (gr)?getProcByGroup(gr->gr_gid):new LinuxProcessReply;
	}

LinuxProcessReply * LinuxProcessList::getProcByParent ( pid_t pid )
	{
	LinuxProcessReply * reply = new LinuxProcessReply();
	LinuxProcess * proc = procs;
	while(proc!=NULL)
		{
		if(proc->parent == pid) reply->addProcess(proc);
		proc = proc->getNext();
		}
	return reply;
	}
	
void LinuxProcessList::asciiDump ( FILE * fp )
	{
	LinuxProcess * proc = procs;
	while(proc!=NULL)
		{
		proc->asciiDump(fp);
		proc = proc->getNext();
		}
	}
