// *****************************************************************************
// * Copyright (c) 1999      Southeastern Universities Research Association,
// *                         Thomas Jefferson National Accelerator Facility
// * 
// * This software was developed under a United States Government license
// * described in the NOTICE file included as part of this distribution.
// *****************************************************************************
// *
// * File:        PBS_MomNode.h
// * Author:      Walt Akers
// * Description: This file contains the class definition for the PBS_MomNode
// *              class.  This class is generally contained by a PBS_Node 
// *              class which provides populates it and provides access to 
// *              its components.
// *****************************************************************************
#ifndef _PBS_MOMNODE_H_
#define _PBS_MOMNODE_H_

// *****************************************************************************
// * class PBS_MomNode :
// *	The PBS_MomNode class is a collection of name/value pairs that are
// *	collected by direct communicatons with the Machine Oriented Mini-Server
// *	on the PBS_Node.  
// *
// *	Typically these values can only be obtained if the requestor is root or
// *	if the host name has been specified as a $restricted host in the 
// *	MOM's config file.
// *****************************************************************************
class PBS_MomNode
{
friend class PBS_Node;

protected:
	enum {DEFAULT_NAME_CNT=13};
	static const char * defaultNameList[DEFAULT_NAME_CNT];
	
	char *  name;
	size_t  entryCnt;
	char ** nameLst;
	char ** valueLst;
	int     errorState;
	
public:
	PBS_MomNode         ( char * nodeName );
	~PBS_MomNode        ( void );
	void   loadData     ( void );
	void   clear        ( void );
	char * findProperty ( char * property );
	int    getProperty  ( char * prop, char * value, int len );
	int    getProperty  ( char * prop, char ** value );
	int    getProperty  ( char * prop, int & value );
	int    getProperty  ( char * prop, unsigned & value );
	int    getProperty  ( char * prop, long & value );
	int    getProperty  ( char * prop, unsigned long & value );
	int    getProperty  ( char * prop, float & value );
	int    getProperty  ( char * prop, double & value );
	int    getMomError  ( void ) { return errorState; }
};

#endif /* _PBS_MOMNODE_H_ */
