// -----------------------------------------------------------------------------
//  Copyright (c) 2000     Southeastern Universities Research Association,
//                         Thomas Jefferson National Accelerator Facility
// 
//  This software was developed under a United States Government license
//  described in the NOTICE file included as part of this distribution.
// -----------------------------------------------------------------------------
// 
//  Project Overview: 
// 	This source code was developed as a component of the UnderLord 
// 	Scheduler for the Parallel Batch System - A joint, wide-area computing
// 	initiative at Jefferson Lab and MIT.
// 
//  Description:
// 	This header defines the PBS_Parameters class that is used to provide
//	system wide information need by applications.
//
//  Author:  Walt Akers
// -----------------------------------------------------------------------------
#ifndef _PBS_PARAMETERS_H_
#define _PBS_PARAMETERS_H_

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/param.h>
#include <PBS_Hash.h>

class PBS_Parameters
{
protected:
	/*************************************************************
	 * ProfileManager:
	 * This is the class that allows the PBS_Parameters object to
	 * read data that is stored in profile databases.
	 *************************************************************/
	 class ProfileManager * profile;
		 
	/*************************************************************
	 * The path to the scheduler configuration files...
	 * This value may be specified in the PBS_CONFIG_DIR variable
	 * or it is /usr/spool/PBS by default.
	 *************************************************************/
	char * configPath;
	
	PBS_Parameters ( void );
	
public:	
	~PBS_Parameters ( void );
			
	static PBS_Parameters & attach ( void );

	void          reload                 ( void );
	const char  * getConfigPath          ( void ) { return (const char *)configPath; }	
	long          getConfigInt           ( char * header, char * entry, long Default );
	double        getConfigFloat         ( char * header, char * entry, double Default );
	const char  * getConfigString        ( char * header, char * entry, const char * Default );
	const char ** getConfigHeaders       ( void );
	const char ** getConfigEntries       ( char * header );
	void          asciiDump              ( FILE * fp = stdout );
};

#endif /* _PBS_PARAMETERS_H_ */
