// -----------------------------------------------------------------------------
//  Copyright (c) 2000     Southeastern Universities Research Association,
//                         Thomas Jefferson National Accelerator Facility
// 
//  This software was developed under a United States Government license
//  described in the NOTICE file included as part of this distribution.
// -----------------------------------------------------------------------------
// 
//  Project Overview: 
// 	This source code was developed as a component of the UnderLord 
// 	Scheduler for the Parallel Batch System - A joint, wide-area computing
// 	initiative at Jefferson Lab and MIT.
// 
//  Description:
//	This class computes the weight of a job based on the job's duration.
// 
//  Author:  Walt Akers
// -----------------------------------------------------------------------------
#ifndef _DURATION_STAGE_H_ 
#define _DURATION_STAGE_H_ 1

#include <UnderLordStage.h>
#include <PBS_Parameters.h>

class DurationStage : public UnderLordStage
{
public:
	inline DurationStage    ( void );
	inline double getWeight ( void );
	void          compute   ( PBS_Job ** jobs, size_t jobCnt, double weightSum );
};

// *****************************************************************************
// * DurationStage::DurationStage :
// *	This is the constructor for the DurationStage. It calls the constructor
// *	for the UnderLordStage object.
// *****************************************************************************
inline DurationStage::DurationStage ( void ) 
	: UnderLordStage ()
	{
	}
	
// *****************************************************************************
// * DurationStage::getWeight :
// *	Returns the weight of this stage in relationship to all other 
// *	stages.
// *****************************************************************************
inline double DurationStage::getWeight ( void )
	{
	double weight = PBS_Parameters::attach().getConfigFloat("Job Duration Stage", "weight", 1.0);
	return (weight<0)?0.0:weight;
	} 
	

#endif /* _DURATION_STAGE_H_ */
