#include <UnderLord.h>
#include <DurationStage.h>
#include <JobAgeStage.h>
#include <UserShareStage.h>
#include <UserPriorityStage.h>
#include <QueuePriorityStage.h>
#include <PBS_Server.h>
#include <JobDeployment.h>
#include <unistd.h>

// *****************************************************************************
// * This variable identifies this application to the logging methods.
// *****************************************************************************
const char * msg_daemonname = SetLogLocal("testMatrix");

class SubUnderLord : public UnderLord
{
public:
	void processJobs ( int connID )
		{
		if(connID>=0 && server.loadData(connID)<0)
			{
			LogError("UnderLord", "Error: Failed to communicate with server\n");
			connID = -1;
			}
		if(connID>=0)
			{
			SchedStatic::startCycle();
			matrix.startCycle(server);
			JobDeployment * scheduled = NULL;
			PBS_Job      ** jobs      = server.getJobsByState(PBS_Job::QUEUED);
			size_t          jobCnt    = 0;

			for(jobCnt=0; jobs && jobs[jobCnt]; jobCnt++);
			sortJobs(server, jobs, jobCnt, 0);
			scheduled = matrix.processJobs(jobs, jobCnt, 0);
			while(scheduled!=NULL)
				{
				scheduled->asciiDump(stdout, NULL);
				scheduled = scheduled->getNext();
				}
			matrix.endCycle();
			SchedStatic::endCycle();
			}
		}

};


int main (int argc, char ** argv)
{
int    connID = -1;
FILE * fp  = NULL;
char   buf[1024];
char * ptr;

if(argc>1)
	{
	connID = pbs_connect(argv[1]);
	}
else if	((fp = fopen("/usr/spool/PBS/server_name", "r"))!=NULL &&
	(fgets(buf, 1024, fp)!=NULL))
	{
	// Remove the trailing carriage return
	char * ptr;
	if((ptr = strchr(buf, '\n'))!=NULL) *ptr = 0;

	connID = pbs_connect(buf);
	fclose(fp);
	}

if(connID<=0)
	{
	fprintf(stdout, "Error: Server name unspecified or not valid\n");
	return 0;
	}

PBS_AccountingDatabase serverDB (ptr); 
SubUnderLord subLord;
subLord.addStage(new DurationStage());
subLord.addStage(new JobAgeStage());
subLord.addStage(new UserPriorityStage());
subLord.addStage(new UserShareStage(serverDB));
subLord.addStage(new QueuePriorityStage(subLord.getServer(), serverDB));
serverDB.update(subLord.getServer());
subLord.processJobs(connID);

return 0;
}
